# -*- coding: utf-8 -*-
# @Time    : 2023/1/22 12:46
# @Author  : Quanfa
"""
path tool
# MyPath
    basic string format representing a path
"""
import os
from os.path import dirname, abspath


def get_files(directory: str, mark='.') -> dict:
    files = {}
    for filename in os.listdir(directory):
        if mark in filename:
            files[filename] = directory + "/" + filename
    return files


class MyPath(str):
    """
    basic class for the path of files (have . at the end) and directories.
    """

    def __new__(cls, path):
        path = str(path)
        if path == '':
            return str.__new__(cls, path)
        path = path.replace('\\', '/')
        if path[-1] == '/':
            path = path[: -1]
        return str.__new__(cls, path)

    def get_level(self, level_start, level_end=None):
        cs = self.split('/')

        if level_end is None:
            return cs[level_start]

        r = ''
        for c in cs[level_start: level_end]:
            r += '/' + c

        return r

    def cat(self, *value):
        result = self
        for other in value:
            if other == '':
                continue
            if other[0] == '/':
                other = MyPath(other[1:])
            else:
                other = MyPath(other)
            if result == '':
                result = other
            else:
                result = result + '/' + other
        result = MyPath(result)
        return result

    def is_file(self):
        name = self.get_name()
        if '.' in name:
            return True
        return False

    def get_name(self):
        return self.split('/')[-1]

    def suffix_of(self):
        if self.is_file():
            return self.split('.')[-1]
        return ''

    def get_parent(self):
        """parent of the file or directory"""
        return MyPath(dirname(self))

    # region existence
    def ensure(self):
        """make sure that the path exist."""
        path = self
        # if path is directory
        if self.is_file():
            path.get_parent().ensure()
            f = open(path, 'a')
            f.close()
        else:
            if not os.path.exists(path):
                os.makedirs(path)

    def exist(self) -> bool:
        return os.path.exists(self)

    # endregion
    @staticmethod
    def from_file(file=__file__):
        return MyPath(abspath(file))

    @staticmethod
    def get_root(file, mask='T20'):
        """ get root using file in .../T20*"""
        path = MyPath.from_file(file)
        return path.my_root(mask)

    def my_root(self, mask='T20'):
        paths = self.split('/')
        path = ''
        for c in paths:
            path += c
            if mask in c:
                break
            path += '/'
        return MyPath(path)
    
    def relative_to(self, mask='T20'):
        path = self.split(mask)[1]
        return MyPath(path)

    def extend_to_sys(self):
        import sys
        sys.path.append(self)

    def get_files(self, mark='.', list_r=False):
        """
        :param mark: file mark, if mark in filename.
        :param list_r: if trans to list
        :return: dict or list
        """
        directory = self
        files = {}
        for filename in os.listdir(directory):
            if mark in filename:
                files[filename] = directory.cat(filename)
        if list_r:
            return list(files.values())
        return files





# -*- coding: utf-8 -*-
# @Time    : 2023/1/12 12:03
# @Author  : Quanfa
# @Desc    :

from .path_tool import MyPath
from .matlab import save_mat, load_mat
import torch as saver
import sys

def save(object, path, name, suffix):
    if suffix == 'Figure':
        object.savefig(path)
    if suffix == 'mat':
        save_mat(object, path, name)
    else:
        saver.save(object, path)
    print('save ',suffix,' at ', path)

class ScriptFileSaver:
    def __init__(self, script_file, locals, version: int=None):
        """
        A combination of database and saver in framework.

        :param root_path: local path
        :param date_mark:
        :param version:
        :param author:
        """
        self.locals = locals
        # region calculate version
        script_path = MyPath.from_file(script_file)
        relative_path = script_path.relative_to('myscripts').get_parent()
        script_name = MyPath.from_file(script_file).get_name()[:-3]
        root_path = script_path.my_root()
        local_path = root_path.cat('mylocal')
        save_path_parent = local_path.cat(relative_path).cat(script_name)
        save_path_parent.ensure()
        if version is None:
            version = len(save_path_parent.get_files(mark='s', list_r=True)) # version exists in save path
            if version == 0:
                version = 1
        # endregion
        self.local_path = save_path_parent.cat(str(version))
        self.version = version
        self.local_path.ensure()
        self.root_path = root_path

        # region append project path to system
        # sys.path.append(root_path.cat('myclasses'))
        # sys.path.append(root_path.cat('myscripts'))
        sys.path.append(root_path)
        # endregion

    def path_of(self, file_name='auto_save_result', suffix='sus'):
        """

        :param file_name:
        :return:
        """
        if suffix == '':
            path = self.local_path.cat(file_name)
        else:
            path = self.local_path.cat(file_name + '.' + suffix)

        return path

    def save(self, name, suffix=None, path=None):
        """
        save object to location.

        :param object:
        :param name:
        :param suffix: sus, sci util saved; mat, matlab
        :return:
        """
        object = self.locals[name]
        if suffix is None:
            suffix = str(type(object)).split("'")[1].split('.')[-1]
        if path is None:
            path = self.path_of(name, suffix)
        else:
            path = MyPath(path)

        save(object, path, suffix, name)
        return path

    def load(self, name=None, suffix=None, object_sample=None, path=None):
        """
        load from specified version.
        :param name:
        :return:
        """
        if path is None:
            if suffix is None:
                path = self.local_path.    get_files(mark=name, list_r=True)[0]
                suffix = path.split('.')[-1]
            else:
                path = self.path_of(name, suffix)
        print('load ',suffix,' from ', path)
        if object_sample is not None:
            return object_sample.load(path)
        if suffix == 'mat':
            return load_mat(path)
        else:
            return saver.load(path)

    # def export(self, names=None, destination: str = None, export_name=None):
    #     """
    #     export saved files to destination.
    #     init params and zip files.
    #     saver scan all files in srcipt file path, so need hide exportation into a folder.
    #
    #     :param names: file names for exportation
    #     :param destination:
    #     :param export_name: export zip name
    #     :return:
    #     """
    #     import os
    #     # param init
    #     if destination is None:
    #         destination = self.local_path
    #     destination.ensure()
    #     if export_name is None:
    #         export_name = self.author + '_' + self.date_mark + self.version + '.zip'
    #     zip_path = destination.cat(export_name)
    #     if names is None:
    #         # zip
    #         with zipfile.ZipFile(zip_path, 'w') as zip:
    #             dirpath = self.__script_file_path
    #             for path, dirnames, filenames in os.walk(dirpath):  # 遍历文件
    #                 fpath = path.split('/mylocal/')[-1]  # 去掉目标跟路径，只对目标文件夹下边的文件及文件夹进行压缩（即生成相对路径）
    #                 for filename in filenames:
    #                     zip.write(os.path.join(path, filename), os.path.join(fpath, filename))
    #
    # def import_from(self, zip_path: str = None):
    #     """
    #     unzip to local is ok.
    #     """
    #     if zip_path is None:
    #         destination = self.local_path
    #         destination.ensure()
    #         export_name = self.author + '_' + self.date_mark + self.version + '.zip'
    #         zip_path = destination.cat(export_name)
    #
    #     with zipfile.ZipFile(zip_path, 'r') as zip:
    #         zip.extractall(self.local_path)
    #
    #     print('import from', zip_path)
    #
    # def end_script(self, __script_file__):
    #     """
    #     print end messages in terminal.
    #
    #     :return:
    #     """
    #     # # get information
    #     # script_file_path = MyPath.from_file(__script_file__)
    #     # script_name = script_file_path.split('/')[-1]
    #     # running_date = self.running_date
    #     # running_time = self.running_version
    #     #
    #     # # renew information
    #     # with open(script_file_path, 'r', encoding="utf-8") as file_old, open(script_file_path+'.tmp', 'w', encoding="utf-8") as file_new:
    #     #     skip = False
    #     #     for line in file_old:
    #     #         if not skip:
    #     #             if 'date = ' in line:
    #     #                 line = 'date = \'' + running_date + '\'\n'
    #     #             if 'version = ' in line:
    #     #                 line = 'version = \''+running_time+'\'\n'
    #     #                 skip = True
    #     #         file_new.write(line)
    #     # os.remove(script_file_path)
    #     # os.rename(script_file_path+'.tmp', script_file_path)
    #     # print
    #     print('script', script_name, 'finished its running in', running_date, 'at', running_time)


# -*- coding: utf-8 -*-
# @Time    : 2023/8/31 11:22
# @Author  : Quanfa
import sys
from .FileSys.project import create_project_at

args = sys.argv[1:]

command = args[0]
print(args)
if command == 'create':
    try:
        project_path = args[1]
        project_name = args[2]
    except:
        print('create [project path] [project name]')

    create_project_at(project_path, project_name)


# -*- coding: utf-8 -*-
# @Time    : 2023/8/31 13:03
# @Author  : Quanfa
from .path_tool import MyPath
def create_project_at(folder=r'D:\Task\a1_Ongoing', name='test'):
    """
    create project and at folder. init then.
    """

    # date, version
    from datetime import datetime
    date = datetime.now().strftime("%Y%m%d")
    project_name = 'T' + date + '_' + name

    init_project(MyPath(folder + '/' + project_name))
    print(folder + '/' + project_name)


# def hang_up_project_(folder=r'D:\Task', name=''):
#     pass


def init_project(root_path: MyPath):
    """
    init project framework in pycharm style
    """
    # init local, myclass, script
    class_path = root_path.cat('/myclasses')
    script_path = root_path.cat('/myscripts')
    report_path = root_path.cat('/myreport')
    local_path = root_path.cat('/mylocal')

    local_path.ensure()
    class_path.ensure()
    script_path.ensure()
    report_path.ensure()

    # init readme.md
    read_file = report_path.cat('/readme.md')
    read_file.ensure()

    # init git.ignore
    git_ignore = root_path.cat('/.gitignore')
    f = open(git_ignore, 'w')
    f.write('.idea\nmylocal\n')

    # set each path

    f = open(script_path + '/__init__.py', 'w')
    f.write(
        """
from EasyPro.FileSys import ScriptFileSaver
sfs = ScriptFileSaver(__file__, locals())

# region src_path
root_path = sfs.root_path
scripts_path = root_path.cat('myscripts')
local_path = root_path.cat('mylocal')
# endregion

if __name__ == '__main__':
    from EasyPro.FileSys.project import create_script
    create_script(scripts_path, name='test')
        """
    )


# def get_date_time():
#     return datetime.now().strftime("d%Y%m%ds%H%M")


# def get_date():
#     return get_date_time()[:9]


# def get_time():
#     return get_date_time()[9:]


def create_script(scripts_path: MyPath, name='test'):
    """
    create script for running or test.

    :param root_path: project path
    :param name: script name
    :return:
    """
    # region script path
    script_dir_path = scripts_path  # script dir

    file_names = script_dir_path.get_files('.py')
    index_files = list(file_names.keys())
    index = len(index_files)

    # build path
    script_path = script_dir_path.cat('s'+str(index)+'_'+ name + '.py')
    script_path: MyPath
    if script_path.exist():
        print('script in this name has already existed in your scripts:',script_path,'Try to use git if you want to edit your script to a new branch.')
        return False
    # endregion

    # write
    f = open(script_path, 'w')
    f.write(
        """# -*- coding: utf-8 -*-
from EasyPro.FileSys import ScriptFileSaver
sfs = ScriptFileSaver(__file__, locals())

if __name__ == '__main__':
    print('Running has finished')
 """
    )
    print('create script at', script_path)

